@@ -138,7 +138,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   /*
     Format caption.
   */
-  option=GetImageArtifact(image,"filename");
+  option=GetImageOption(image_info,"filename");
   if (option == (const char *) NULL)
     property=InterpretImageProperties(image_info,image,image_info->filename);
   else
@@ -151,7 +151,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   caption=ConstantString(GetImageProperty(image,"caption"));
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
   (void) CloneString(&draw_info->text,caption);
-  gravity=GetImageArtifact(image,"gravity");
+  gravity=GetImageOption(image_info,"gravity");
   if (gravity != (char *) NULL)
     draw_info->gravity=(GravityType) ParseCommandOption(MagickGravityOptions,
       MagickFalse,gravity);
@@ -206,10 +206,15 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
         (void) status;
         width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
         height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
-        if ((width >= image->columns) && (height >= image->rows))
-          break;
-        if ((width >= (image->columns << 1)) || (height >= (image->rows << 1)))
-          break;
+        if ((image->columns != 0) && (image->rows != 0))
+          {
+            if ((width > image->columns) && (height > image->rows))
+              break;
+          }
+        else
+          if (((image->columns != 0) && (width > image->columns)) ||
+              ((image->rows != 0) && (height > image->rows)))
+            break;
       }
       high=draw_info->pointsize;
       for (low=1.0; (high-low) > 1.0; )
@@ -226,29 +231,21 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
         status=GetMultilineTypeMetrics(image,draw_info,&metrics);
         width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
         height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
-        if ((width <= image->columns) && (height <= image->rows))
-          low=draw_info->pointsize+1.0;
+        if ((image->columns != 0) && (image->rows != 0))
+          {
+            if ((width <= image->columns) && (height <= image->rows))
+              low=draw_info->pointsize+1.0;
+            else
+              high=draw_info->pointsize-1.0;
+          }
         else
-          high=draw_info->pointsize-1.0;
+          if (((image->columns != 0) && (width < image->columns)) ||
+              ((image->rows != 0) && (height < image->rows)))
+            low=draw_info->pointsize+1.0;
+          else
+            high=draw_info->pointsize-1.0;
       }
-      for (draw_info->pointsize=(low+high)/2.0; (high-low) > 1.0; )
-      {
-        text=AcquireString(caption);
-        i=FormatMagickCaption(image,draw_info,MagickFalse,&metrics,&text);
-        (void) CloneString(&draw_info->text,text);
-        text=DestroyString(text);
-        (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
-          -metrics.bounds.x1,metrics.ascent);
-        if (draw_info->gravity == UndefinedGravity)
-          (void) CloneString(&draw_info->geometry,geometry);
-        status=GetMultilineTypeMetrics(image,draw_info,&metrics);
-        width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
-        height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
-        if ((width <= image->columns) && (height <= image->rows))
-          break;
-        draw_info->pointsize--;
-      }
-      draw_info->pointsize--;
+      draw_info->pointsize=(low+high)/2.0;
     }
   (void) CloneString(&draw_info->text,caption);
   i=FormatMagickCaption(image,draw_info,MagickFalse,&metrics,&caption);
