@@ -157,12 +157,15 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
         (void) status;
         width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
         height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
-        if (((image->columns != 0) && (width >= image->columns)) ||
-            ((image->rows != 0) && (height >= image->rows)))
-          break;
-        if (((image->columns != 0) && (width >= (2*image->columns))) ||
-            ((image->rows != 0) && (height >= (2*image->rows))))
-          break;
+        if ((image->columns != 0) && (image->rows != 0))
+          {
+            if ((width > image->columns) && (height > image->rows))
+              break;
+          }
+        else
+          if (((image->columns != 0) && (width > image->columns)) ||
+              ((image->rows != 0) && (height > image->rows)))
+            break;
       }
       high=draw_info->pointsize;
       for (low=1.0; (high-low) > 1.0; )
@@ -175,27 +178,21 @@ static Image *ReadLABELImage(const ImageInfo *image_info,
         status=GetMultilineTypeMetrics(image,draw_info,&metrics);
         width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
         height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
-        if (((image->columns != 0) && (width < image->columns)) ||
-            ((image->rows != 0) && (height < image->rows)))
-          low=draw_info->pointsize+1.0;
+        if ((image->columns != 0) && (image->rows != 0))
+          {
+            if ((width <= image->columns) && (height <= image->rows))
+              low=draw_info->pointsize+1.0;
+            else
+              high=draw_info->pointsize-1.0;
+          }
         else
-          high=draw_info->pointsize-1.0;
+          if (((image->columns != 0) && (width < image->columns)) ||
+              ((image->rows != 0) && (height < image->rows)))
+            low=draw_info->pointsize+1.0;
+          else
+            high=draw_info->pointsize-1.0;
       }
-      for (draw_info->pointsize=(low+high)/2.0; (high-low) > 1.0; )
-      {
-        (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
-          -metrics.bounds.x1,metrics.ascent);
-        if (draw_info->gravity == UndefinedGravity)
-          (void) CloneString(&draw_info->geometry,geometry);
-        status=GetMultilineTypeMetrics(image,draw_info,&metrics);
-        width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
-        height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
-        if (((image->columns != 0) && (width < image->columns)) ||
-            ((image->rows != 0) && (height < image->rows)))
-          break;
-        draw_info->pointsize--;
-      }
-      draw_info->pointsize--;
+      draw_info->pointsize=(low+high)/2.0;
     }
   status=GetMultilineTypeMetrics(image,draw_info,&metrics);
   if (status == MagickFalse)
