@@ -547,6 +547,13 @@ ModuleExport size_t RegisterTXTImage(void)
   MagickInfo
     *entry;
 
+  entry=SetMagickInfo("SPARSE-COLOR");
+  entry->encoder=(EncodeImageHandler *) WriteTXTImage;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
+  entry->description=ConstantString("Sparse Color");
+  entry->module=ConstantString("TXT");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TEXT");
   entry->decoder=(DecodeImageHandler *) ReadTXTImage;
   entry->encoder=(EncodeImageHandler *) WriteTXTImage;
@@ -663,11 +670,14 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
     image->depth=GetImageQuantumDepth(image,MagickTrue);
     if (image->matte != MagickFalse)
       (void) ConcatenateMagickString(colorspace,"a",MaxTextExtent);
-    (void) FormatLocaleString(buffer,MaxTextExtent,
-      "# ImageMagick pixel enumeration: %.20g,%.20g,%.20g,%s\n",(double)
-      image->columns,(double) image->rows,(double)
-      ((MagickOffsetType) GetQuantumRange(image->depth)),colorspace);
-    (void) WriteBlobString(image,buffer);
+    if (LocaleCompare(image_info->magick,"SPARSE-COLOR") != 0)
+      {
+        (void) FormatLocaleString(buffer,MaxTextExtent,
+          "# ImageMagick pixel enumeration: %.20g,%.20g,%.20g,%s\n",(double)
+          image->columns,(double) image->rows,(double)
+          ((MagickOffsetType) GetQuantumRange(image->depth)),colorspace);
+        (void) WriteBlobString(image,buffer);
+      }
     GetMagickPixelPacket(image,&pixel);
     for (y=0; y < (ssize_t) image->rows; y++)
     {
@@ -677,15 +687,34 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
       indexes=GetVirtualIndexQueue(image);
       for (x=0; x < (ssize_t) image->columns; x++)
       {
-        (void) FormatLocaleString(buffer,MaxTextExtent,"%.20g,%.20g: ",(double)
-          x,(double) y);
-        (void) WriteBlobString(image,buffer);
         SetMagickPixelPacket(image,p,indexes+x,&pixel);
         if (pixel.colorspace == LabColorspace)
           {
             pixel.green-=(QuantumRange+1)/2.0;
             pixel.blue-=(QuantumRange+1)/2.0;
           }
+        if (LocaleCompare(image_info->magick,"SPARSE-COLOR") == 0)
+          {
+            /*
+              Sparse-color format.
+            */
+            if (GetPixelOpacity(p) == (Quantum) OpaqueOpacity)
+              {
+                GetColorTuple(&pixel,MagickFalse,tuple);
+                (void) QueryMagickColorname(image,&pixel,SVGCompliance,tuple,
+                  &image->exception);
+                (void) FormatLocaleString(buffer,MaxTextExtent,"%.20g,%.20g,",
+                  (double) x,(double) y);
+                (void) WriteBlobString(image,buffer);
+                (void) WriteBlobString(image,tuple);
+                (void) WriteBlobString(image," ");
+              }
+            p++;
+            continue;
+          }
+        (void) FormatLocaleString(buffer,MaxTextExtent,"%.20g,%.20g: ",(double)
+          x,(double) y);
+        (void) WriteBlobString(image,buffer);
         (void) CopyMagickString(tuple,"(",MaxTextExtent);
         ConcatenateColorComponent(&pixel,RedChannel,X11Compliance,tuple);
         (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
