@@ -46,6 +46,7 @@
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/cache.h"
+#include "magick/cache-private.h"
 #include "magick/cache-view.h"
 #include "magick/client.h"
 #include "magick/channel.h"
@@ -723,7 +724,7 @@ MagickExport MagickBooleanType IsGrayImage(const Image *image,
       (image->type == GrayscaleMatteType))
     return(MagickTrue);
   if ((IsGrayColorspace(image->colorspace) == MagickFalse) &&
-      (IsRGBColorspace(image->colorspace) == MagickFalse))
+      (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse))
     return(MagickFalse);
   type=BilevelType;
   image_view=AcquireVirtualCacheView(image,exception);
@@ -749,10 +750,11 @@ MagickExport MagickBooleanType IsGrayImage(const Image *image,
   image_view=DestroyCacheView(image_view);
   if (type == UndefinedType)
     return(MagickFalse);
+  ((Image *) image)->colorspace=GRAYColorspace;
   ((Image *) image)->type=type;
   if ((type == GrayscaleType) && (image->matte != MagickFalse))
     ((Image *) image)->type=GrayscaleMatteType;
-  return(MagickTrue);
+  return(SyncImagePixelCache((Image *) image,exception));
 }
 
 /*
@@ -807,7 +809,7 @@ MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
   if (image->type == BilevelType)
     return(MagickTrue);
   if ((IsGrayColorspace(image->colorspace) == MagickFalse) &&
-      (IsRGBColorspace(image->colorspace) == MagickFalse))
+      (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse))
     return(MagickFalse);
   type=BilevelType;
   image_view=AcquireVirtualCacheView(image,exception);
@@ -831,8 +833,9 @@ MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
   image_view=DestroyCacheView(image_view);
   if (type == UndefinedType)
     return(MagickFalse);
+  ((Image *) image)->colorspace=GRAYColorspace;
   ((Image *) image)->type=type;
-  return(MagickTrue);
+  return(SyncImagePixelCache((Image *) image,exception));
 }
 
 /*
