@@ -17,7 +17,7 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2013 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,12 +40,14 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/cache.h"
 #include "magick/client.h"
+#include "magick/colorspace-private.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
 #include "magick/geometry.h"
@@ -1099,6 +1101,14 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
             image->endian=(*(char *) &lsb_first) == 1 ? LSBEndian : MSBEndian;
          }
     }
+  if (IsGrayColorspace(image->colorspace) != MagickFalse)
+    {
+      /*
+        sRGB masquerading as a grayscale image?
+      */
+      if (IsGrayImage(image,&image->exception) == MagickFalse)
+        (void) SetImageColorspace(image,sRGBColorspace);
+    }
   (void) SyncImageProfiles(image);
   option=GetImageOption(image_info,"delegate:bimodal");
   if ((option != (const char *) NULL) &&
@@ -1219,11 +1229,11 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
                   (GetImageEncoder(magick_info) == (EncodeImageHandler *) NULL))
                 (void) ThrowMagickException(&image->exception,GetMagickModule(),
                   MissingDelegateError,"NoEncodeDelegateForThisImageFormat",
-                  "'%s'",image->filename);
+                  "`%s'",image->filename);
               else
                 (void) ThrowMagickException(&image->exception,GetMagickModule(),
                   MissingDelegateWarning,"NoEncodeDelegateForThisImageFormat",
-                  "'%s'",image->filename);
+                  "`%s'",image->filename);
             }
           if ((magick_info != (const MagickInfo *) NULL) &&
               (GetImageEncoder(magick_info) != (EncodeImageHandler *) NULL))
