@@ -109,7 +109,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   const MagickPixelPacket *target,const ssize_t x_offset,const ssize_t y_offset,
   const MagickBooleanType invert)
 {
-#define MaxStacksize  (1UL << 15)
+#define MaxStacksize  131072UL
 #define PushSegmentStack(up,left,right,delta) \
 { \
   if (s >= (segment_stack+MaxStacksize)) \
@@ -602,8 +602,6 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
         linear_image=DestroyImage(paint_image);
       return((Image *) NULL);
     }
-  if (image->colorspace == sRGBColorspace)
-    (void) TransformImageColorspace(linear_image,RGBColorspace);
   if (SetImageStorageClass(paint_image,DirectClass) == MagickFalse)
     {
       InheritException(exception,&paint_image->exception);
@@ -688,8 +686,8 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       {
         for (u=0; u < (ssize_t) width; u++)
         {
-          k=(ssize_t) ScaleQuantumToChar(PixelIntensityToQuantum(linear_image,
-            p+u+i));
+          k=(ssize_t) ScaleQuantumToChar(ClampToQuantum(GetPixelIntensity(
+            linear_image,p+u+i)));
           histogram[k]++;
           if (histogram[k] > count)
             {
@@ -724,8 +722,6 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
   image_view=DestroyCacheView(image_view);
   histograms=DestroyHistogramThreadSet(histograms);
   linear_image=DestroyImage(linear_image);
-  if (image->colorspace == sRGBColorspace)
-    (void) TransformImageColorspace(paint_image,sRGBColorspace);
   if (status == MagickFalse)
     paint_image=DestroyImage(paint_image);
   return(paint_image);
@@ -815,7 +811,7 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
     return(MagickFalse);
   if ((IsGrayColorspace(image->colorspace) != MagickFalse) &&
       (IsMagickGray(fill) == MagickFalse))
-    (void) TransformImageColorspace(image,RGBColorspace);
+    (void) SetImageColorspace(image,sRGBColorspace);
   if ((fill->opacity != OpaqueOpacity) && (image->matte == MagickFalse))
     (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
   /*
