@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2013 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -22,6 +22,11 @@
 extern "C" {
 #endif
 
+#include "magick/draw.h"
+#include "magick/exception.h"
+#include "magick/geometry.h"
+#include "magick/quantize.h"
+
 #if defined(MAGICKCORE_X11_DELEGATE)
 
 #include <X11/Xos.h>
@@ -31,9 +36,6 @@ extern "C" {
 #include <X11/keysym.h>
 #include <X11/Xresource.h>
 #include <X11/Xutil.h>
-#include "magick/exception.h"
-#include "magick/geometry.h"
-#include "magick/quantize.h"
 
 #if defined(__cplusplus) || defined(c_plusplus)
 # define klass  c_class
@@ -591,12 +593,16 @@ static inline MagickRealType XPixelIntensity(const XColor *pixel)
   MagickRealType
     intensity;
 
-  intensity=0.299*pixel->red+0.587*pixel->green+0.114*pixel->blue;
+  if ((pixel->red  == pixel->green) && (pixel->green == pixel->blue))
+    return((MagickRealType) pixel->red);
+  intensity=0.298839*pixel->red+0.586811*pixel->green+0.114350*pixel->blue;
   return(intensity);
 }
-
 #endif
 
+extern MagickPrivate MagickBooleanType
+  XRenderImage(Image *,const DrawInfo *,const PointInfo *,TypeMetric *);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
